"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var utils_1 = require("../utils");
describe(__filename + "#", function () {
    it("can render a simple style", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, text, _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n        .a {\n          color: b;\n        }\n      </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    _a = utils_1.stringifyLoadResult;
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    text = _a.apply(void 0, [_b.sent()]);
                    chai_1.expect(text).to.eql("<style>[class]._80f4925f_a { color:b; }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("displays an error if style url not found", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, e, err;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "\n        <style>\n          .rule {\n            background: url('/not/found.png')\n          }\n        </style>\n      "
                    };
                    engine = utils_1.createMockEngine(graph, function () { }, {
                        resolveFile: function (uri) {
                            return null;
                        }
                    });
                    e = utils_1.waitForError(engine);
                    engine.run("/entry.pc");
                    return [4 /*yield*/, e];
                case 1:
                    err = _a.sent();
                    chai_1.expect(err).to.eql({
                        kind: "Error",
                        errorKind: "Runtime",
                        uri: "/entry.pc",
                        location: { start: 59, end: 91 },
                        message: "Unable to resolve file."
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    describe("Mixins", function () {
        it("can be created & used", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, text, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<style>\n          @mixin a {\n            color: blue;\n          }\n          div {\n            @include a;\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        _a = utils_1.stringifyLoadResult;
                        return [4 /*yield*/, engine.run("/entry.pc")];
                    case 1:
                        text = _a.apply(void 0, [_b.sent()]);
                        chai_1.expect(text).to.eql("<style>div[data-pc-80f4925f] { color:blue; }</style>");
                        return [2 /*return*/];
                }
            });
        }); });
        it("Displays an error if a mixin is not found", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, p, e;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<style>\n          div {\n            @include a;\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        p = utils_1.waitForError(engine);
                        engine.run("/entry.pc").catch(function () { });
                        return [4 /*yield*/, p];
                    case 1:
                        e = _a.sent();
                        chai_1.expect(e).to.eql({
                            kind: "Error",
                            errorKind: "Runtime",
                            uri: "/entry.pc",
                            location: { start: 45, end: 46 },
                            message: "Reference not found."
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it("can use an imported mixin", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, text, _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<import as=\"mod\" src=\"./module.pc\"><style>\n          div {\n            @include mod.a;\n          }\n        </style>",
                            "/module.pc": "<style>\n          @export {\n            @mixin a {\n              color: orange;\n            }\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        _a = utils_1.stringifyLoadResult;
                        return [4 /*yield*/, engine.run("/entry.pc")];
                    case 1:
                        text = _a.apply(void 0, [_b.sent()]);
                        chai_1.expect(text).to.eql("<style>div[data-pc-80f4925f] { color:orange; }</style>");
                        return [2 /*return*/];
                }
            });
        }); });
        it("Displays an error if an imported mixin is not found", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, p, e;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<import as=\"mod\" src=\"./module.pc\"><style>\n          div {\n            @include mod.a;\n          }\n        </style>",
                            "/module.pc": "<style>\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        p = utils_1.waitForError(engine);
                        engine.run("/entry.pc").catch(function () { });
                        return [4 /*yield*/, p];
                    case 1:
                        e = _a.sent();
                        chai_1.expect(e).to.eql({
                            kind: "Error",
                            errorKind: "Runtime",
                            uri: "/entry.pc",
                            location: { start: 84, end: 85 },
                            message: "Reference not found."
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it("Displays an error if the import is not found", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, p, e;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<style>\n          div {\n            @include mod.a;\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        p = utils_1.waitForError(engine);
                        engine.run("/entry.pc").catch(function () { });
                        return [4 /*yield*/, p];
                    case 1:
                        e = _a.sent();
                        chai_1.expect(e).to.eql({
                            kind: "Error",
                            errorKind: "Runtime",
                            uri: "/entry.pc",
                            location: { start: 45, end: 48 },
                            message: "Reference not found."
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        // expectation is still incorrect, just want to make sure that this doesn't break the engine
        it("Smoke -- can use nested refs", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, p, e;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<style>\n          div {\n            @include a.b.c;\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        p = utils_1.waitForError(engine);
                        engine.run("/entry.pc").catch(function () { });
                        return [4 /*yield*/, p];
                    case 1:
                        e = _a.sent();
                        return [2 /*return*/];
                }
            });
        }); });
        it("Displays an error if a mixin is used but not exported", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, p, e;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<import as=\"mod\" src=\"./module.pc\"><style>\n          div {\n            @include mod.abcde;\n          }\n        </style>",
                            "/module.pc": "<style>\n          @mixin abcde {\n            color: orange;\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        p = utils_1.waitForError(engine);
                        engine.run("/entry.pc").catch(function () { });
                        return [4 /*yield*/, p];
                    case 1:
                        e = _a.sent();
                        chai_1.expect(e).to.eql({
                            kind: "Error",
                            errorKind: "Runtime",
                            uri: "/entry.pc",
                            location: { start: 84, end: 89 },
                            message: "This mixin is private."
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it("Display an error if a mixins is already defined in the upper scope", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, p, e;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<style>\n          @mixin abcde {\n            color: blue;\n          }\n          \n          @mixin abcde {\n            color: orange;\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        p = utils_1.waitForError(engine);
                        engine.run("/entry.pc").catch(function () { });
                        return [4 /*yield*/, p];
                    case 1:
                        e = _a.sent();
                        chai_1.expect(e).to.eql({
                            kind: "Error",
                            errorKind: "Runtime",
                            uri: "/entry.pc",
                            location: { start: 98, end: 103 },
                            message: "This mixin is already declared in the upper scope."
                        });
                        return [2 /*return*/];
                }
            });
        }); });
        it("properly concats using multiple &", function () { return __awaiter(void 0, void 0, void 0, function () {
            var graph, engine, result;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        graph = {
                            "/entry.pc": "<style>\n          .company_list {\n            list-style: none;\n            margin: 0;\n            padding: 0;\n        \n            & li {\n        \n              display: block;\n              padding: var(--spacing-600) 0;\n        \n              & + & {\n                border-top: 1px solid var(--color-black-100);\n              }\n            }\n          }\n        </style>"
                        };
                        engine = utils_1.createMockEngine(graph);
                        return [4 /*yield*/, engine.run("/entry.pc")];
                    case 1:
                        result = _a.sent();
                        chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>[class]._80f4925f_company_list { list-style:none; margin:0; padding:0; } [class]._80f4925f_company_list li[data-pc-80f4925f] { display:block; padding:var(--spacing-600) 0; } [class]._80f4925f_company_list li[data-pc-80f4925f] + [class]._80f4925f_company_list li[data-pc-80f4925f] { border-top:1px solid var(--color-black-100); }</style>");
                        return [2 /*return*/];
                }
            });
        }); });
    });
    it("can use escape key in classname", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n        .a\\:b {\n        }\n      </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>[class]._80f4925f_a\\:b { }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("can use single line comment", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n\n\n      // :checked\n      input:checked {\n        & + .tab-label {\n          background: var(--midnight-darker);\n          &::after {\n            transform: rotate(90deg);\n          }\n        }\n        & ~ .tab-content {\n          max-height: 100vh;\n          padding: 1em;\n        }\n      }\n      \n      </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>input:checked[data-pc-80f4925f] { } input:checked[data-pc-80f4925f] + [class]._80f4925f_tab-label { background:var(--midnight-darker); } input:checked[data-pc-80f4925f] + [class]._80f4925f_tab-label::after { transform:rotate(90deg); } input:checked[data-pc-80f4925f] ~ [class]._80f4925f_tab-content { max-height:100vh; padding:1em; }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("errors if comment is unterminated", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, err, result, e_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n        /* foreverrrrrr\n      </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 2:
                    result = _a.sent();
                    return [3 /*break*/, 4];
                case 3:
                    e_1 = _a.sent();
                    err = e_1;
                    return [3 /*break*/, 4];
                case 4:
                    chai_1.expect(err).to.eql({
                        errorKind: "Graph",
                        uri: "/entry.pc",
                        info: {
                            kind: "Unterminated",
                            message: "Unterminated element.",
                            location: { start: 0, end: 7 }
                        }
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    it("CSS vars are collected in the evaluated output", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n        .element {\n          --color: test;\n        }\n      </style>ab"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(result.exports.style.variables["--color"]).to.eql({
                        name: "--color",
                        value: "test",
                        source: {
                            uri: "/entry.pc",
                            location: {
                                start: 37,
                                end: 51
                            }
                        }
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    it("CSS class names are pulled out", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n\n        // comment test\n        [a] {\n          & .color {\n\n          }\n        }\n        @export {\n          .div {\n\n          }\n        }\n        .element {\n          &.child {\n\n          }\n          &--child {\n\n          }\n        }\n      </style>ab"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(result.exports.style.classNames).to.eql({
                        color: { name: "color", public: false },
                        div: { name: "div", public: true },
                        child: { name: "child", public: false },
                        "element--child": { name: "element--child", public: false },
                        element: { name: "element", public: false }
                    });
                    return [2 /*return*/];
            }
        });
    }); });
    it("maintains space with & selector", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n      .todo {\n        &:hover .destroy {\n            display: inline-block;\n        }\n        & .todo {\n          &--item .destroy {\n            display: inline-block;\n          }\n        }\n      }\n    </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>[class]._80f4925f_todo { } [class]._80f4925f_todo:hover [class]._80f4925f_destroy { display:inline-block; } [class]._80f4925f_todo [class]._80f4925f_todo { } [class]._80f4925f_todo [class]._80f4925f_todo--item [class]._80f4925f_destroy { display:inline-block; }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("can parse nested tag selectors", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n      a {\n        & svg:a {\n          margin-right: 4px;\n        }\n      }\n    </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>a[data-pc-80f4925f] { } a[data-pc-80f4925f] svg:a[data-pc-80f4925f] { margin-right:4px; }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("can render keyframes with a dash in the name", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result, ast;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n      @keyframes lds-something3 {\n\n      }\n\n      div {\n        animation: lds-something3 1s;\n      }\n    </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    return [4 /*yield*/, engine.getLoadedAst("/entry.pc")];
                case 2:
                    ast = _a.sent();
                    chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>@keyframes _80f4925f_lds-something3 { } div[data-pc-80f4925f] { animation:_80f4925f_lds-something3 1s; }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("can evaluated multiple nested selectors without &", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result, ast;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n      a {\n        > b {\n          \n        }\n        + c {\n          \n        }\n        ~ d {\n          \n        }\n        :not(.div) {\n\n        }\n        ::active {\n\n        }\n      }\n    </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    return [4 /*yield*/, engine.getLoadedAst("/entry.pc")];
                case 2:
                    ast = _a.sent();
                    chai_1.expect(utils_1.stringifyLoadResult(result)).to.eql("<style>a[data-pc-80f4925f] { } a[data-pc-80f4925f] > b[data-pc-80f4925f] { } a[data-pc-80f4925f] + c[data-pc-80f4925f] { } a[data-pc-80f4925f] ~ d[data-pc-80f4925f] { } a[data-pc-80f4925f] [data-pc-80f4925f]:not([class]._80f4925f_div) { } a[data-pc-80f4925f] [data-pc-80f4925f]::active { }</style>");
                    return [2 /*return*/];
            }
        });
    }); });
    it("includes keyframes in export", function () { return __awaiter(void 0, void 0, void 0, function () {
        var graph, engine, result;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    graph = {
                        "/entry.pc": "<style>\n      @keyframes a {\n\n      }\n      @export {\n        @keyframes b {\n\n        }\n      }\n\n    </style>"
                    };
                    engine = utils_1.createMockEngine(graph);
                    return [4 /*yield*/, engine.run("/entry.pc")];
                case 1:
                    result = _a.sent();
                    chai_1.expect(result.exports.style.keyframes).to.eql({
                        b: {
                            name: "b",
                            public: true,
                            source: {
                                uri: "/entry.pc",
                                location: {
                                    start: 73,
                                    end: 94
                                }
                            }
                        },
                        a: {
                            name: "a",
                            public: false,
                            source: {
                                uri: "/entry.pc",
                                location: {
                                    start: 25,
                                    end: 44
                                }
                            }
                        }
                    });
                    return [2 /*return*/];
            }
        });
    }); });
});
