"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var __1 = require("../../..");
var chai_1 = require("chai");
describe(__filename + "#", function () {
    var cases = [
        [
            "{a}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "{a.b}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: false,
                            properties: {
                                b: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "{{a, b, c: [d, {e, f: b.d}]}}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    b: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: false,
                            properties: {
                                d: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                }
                            }
                        }
                    },
                    d: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    e: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "{a.b} {a.c}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: false,
                            properties: {
                                b: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                },
                                c: {
                                    optional: false,
                                    value: {
                                        kind: 2
                                    }
                                }
                            }
                        }
                    }
                }
            }
        ],
        [
            "<span {a}></span>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<span a={a}></span>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<span {...a}></span>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 0,
                            fromSpread: true,
                            properties: {}
                        }
                    }
                }
            }
        ],
        [
            "<div a={<div>{a}</div>}></div>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<div component as=\"a\">\n        {a}\n      </div>{b}",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    a: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    },
                    b: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<div component as=\"a\" class:test>\n      </div>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    test: {
                        optional: true,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ],
        [
            "<div component as=\"a\" class:test>\n        {test}\n      </div>",
            {
                kind: 0,
                fromSpread: false,
                properties: {
                    test: {
                        optional: false,
                        value: {
                            kind: 2
                        }
                    }
                }
            }
        ]
    ];
    var engine = new __1.Engine({}, function () { });
    var _loop_1 = function (source, inference) {
        it("can infer " + source, function () {
            var ast = engine.parseContent(String(source));
            chai_1.expect(__1.infer(ast)).to.eql(inference);
        });
    };
    for (var _i = 0, cases_1 = cases; _i < cases_1.length; _i++) {
        var _a = cases_1[_i], source = _a[0], inference = _a[1];
        _loop_1(source, inference);
    }
});
