"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveAllPaperclipFiles = void 0;
var path = require("path");
var glob = require("glob");
var paperclip_utils_1 = require("paperclip-utils");
// TODO - move to paperclip-utils as soon as we have a glob library that can handle virtual file systems
exports.resolveAllPaperclipFiles = function (fs) { return function (fromUri, relative) {
    var fromPath = new URL(fromUri).pathname;
    var fromPathDirname = path.dirname(fromPath);
    var configUrl = paperclip_utils_1.findPCConfigUrl(fs)(fromUri);
    // If there's no config, then don't bother looking for PC files. Otherwise we're likely
    // to need esoteric logic for resolving PC that I don't think should be supported -- there should
    // just be aproach.
    if (!configUrl) {
        return [];
    }
    var configPath = new URL(configUrl).pathname;
    var config = JSON.parse(fs.readFileSync(configPath, "utf8"));
    return glob
        .sync(config.filesGlob, { cwd: path.dirname(configPath), realpath: true })
        .filter(function (pathname) { return pathname !== fromPath; })
        .map(function (pathname) {
        if (relative) {
            var modulePath = getModulePath(configUrl, config, pathname, fromPathDirname);
            if (modulePath.charAt(0) !== "/") {
                return modulePath;
            }
            var relativePath = path.relative(fromPathDirname, modulePath);
            if (relativePath.charAt(0) !== ".") {
                relativePath = "./" + relativePath;
            }
            return relativePath;
        }
        return "file://" + pathname;
    });
}; };
var getModulePath = function (configUri, config, modulePath, fromDir) {
    var configDir = path.dirname(new URL(configUri).pathname);
    for (var _i = 0, _a = config.moduleDirectories || []; _i < _a.length; _i++) {
        var moduleRelativeDirectory = _a[_i];
        //
        var moduleDirectory = path.join(configDir, moduleRelativeDirectory) + "/";
        if (modulePath.indexOf(moduleDirectory) === 0) {
            var relativePath = modulePath.replace(moduleDirectory, "");
            var nextDirectory = path.join(moduleDirectory, relativePath.split("/")[0]) + "/";
            if (!fromDir || fromDir.indexOf(nextDirectory) === -1) {
                return relativePath;
            }
        }
    }
    return modulePath;
};
